{-# language RecordWildCards #-}

module Fedi.Crypto where

import Crypto.Hash qualified as Crypto
import Crypto.PubKey.RSA.PSS qualified as Crypto
import Crypto.Store.X509 qualified as Crypto
import Crypto.Store.PKCS8 qualified as Crypto
import Data.X509 qualified as Crypto
import Fedi.Helpers
import Fedi.UserDetails
import Data.ByteString.Base64 qualified as Base64
import Data.Base64.Types qualified as Base64

verifyPub :: ByteString -> ByteString -> ByteString -> Bool
verifyPub pubkeypem sig message =
  let
    pubkey = case Crypto.readPubKeyFileFromMemory pubkeypem of
      [Crypto.PubKeyRSA pubkey'] -> pubkey'
      _ -> error "failed to read pubkey pem"
  in
    Crypto.verify (Crypto.defaultPSSParams Crypto.SHA256) pubkey message sig

verifyDigest :: ByteString -> ByteString -> ByteString -> Bool
verifyDigest pubkeypem sig digest' =
  let
    pubkey = case Crypto.readPubKeyFileFromMemory pubkeypem of
      [Crypto.PubKeyRSA pubkey'] -> pubkey'
      _ -> error "failed to read pubkey pem"
    digest = Crypto.hash digest'
  in
    Crypto.verifyDigest (Crypto.defaultPSSParams Crypto.SHA256) pubkey digest sig

sign :: UserDetails -> ByteString -> IO Signed
sign details message = do
  -- get private key
  privkeypem <- Crypto.readKeyFile details.privatePem
  let
    privateKey = case privkeypem of
      [Crypto.Unprotected (Crypto.PrivKeyRSA privkey)] -> privkey
      _ -> error $ "error reading local private key from '" <> details.privatePem <> "'."

  -- sign message
  signedMessage <- either (error . show) id <$>
    Crypto.sign Nothing (Crypto.defaultPSSParams Crypto.SHA256) privateKey message

  -- sign digest
  let
    digest :: Crypto.Digest Crypto.SHA256
    digest = Crypto.hash message

  signedDigest <- either (error . show) id <$>
    Crypto.signDigest Nothing (Crypto.defaultPSSParams Crypto.SHA256) privateKey digest

  -- return
  pure Signed{..}

data Signed
  = Signed
    { signedMessage :: ByteString
    , signedDigest :: ByteString
    }
  deriving Show

ppSigned :: Signed -> String
ppSigned signed =
  unlines
    [ "Signature"
    , "{ signedMessage = " <> encodeBase64 signed.signedMessage
    , ", signedDigest = " <> encodeBase64 signed.signedDigest
    , "}"
    ]

encodeBase64 :: ByteString -> String
encodeBase64 = show . Base64.extractBase64 . Base64.encodeBase64
